<?php
/**
 * ownCloud
 *
 * @author Owen Winkler
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

\OCP\JSON::checkAdminUser();
\OCP\JSON::checkAppEnabled('enterprise_key');
\OCP\JSON::callCheck();

$sensitiveServerConfigs = [
	'HTTP_COOKIE',
	'PATH',
	'Cookie',
	'include_path',
];

function isKeyDisallowed($key) {
	$sensitiveConfigs = [
		'secret',
		'passwordsalt',
		'dbpassword',
		'dbuser',
		'mail_smtppassword',
		'mail_smtpname',
		'ldap_agent_password'
	];

	if (in_array($key, $sensitiveConfigs)) {
		return true;
	}

	// filter all key which contain 'password'
	if (stripos($key, 'password') !== FALSE) {
		return true;
	}
	return false;
}

$l = OC_L10N::get('enterprise_key');

$error = false;

// Basic report data
$userids = \OCP\User::getUsers();
$homecount = 0;
foreach($userids as $uid) {
	if(OC_User::getManager()->get($uid)) {
		$homecount++;
	}
}
$report = array(
	'basic' => array(
		'license key' => \OCP\Config::getSystemValue('license-key'),
		'date' => date('r'),
		'ownCloud version' => implode('.', OC_Util::getVersion()),
		'ownCloud version string' => OC_Util::getVersionString(),
		'ownCloud edition' => OC_Util::getEditionString(),
		'server OS' => PHP_OS,
		'server OS version' => php_uname(),
		'server SAPI' => php_sapi_name(),
		'webserver version' => $_SERVER['SERVER_SOFTWARE'],
		'hostname' => $_SERVER['HTTP_HOST'],
		'user count' => count($userids),
		'user directories' => $homecount,
		'logged-in user' => \OCP\User::getDisplayName(),
	)
);

// Get app data
$apps = OC_App::listAllApps();
foreach($apps as &$app) {
	if($app['active']) {
		$appConfig = \OC::$server->getAppConfig()->getValues($app['id'], false);
		foreach($appConfig as $key => $value) {
			if (isKeyDisallowed($key) ) {
				$appConfig[$key] = 'xxxx Value is set but was removed by reporting app for security reasons xxxx';
			}
		}
		$app['appconfig'] = $appConfig;
	}
}
$report['apps'] = $apps;

// Get the OC config and filter out the sensitive bits
$keys = OC_Config::getKeys();
foreach ($keys as $key) {
	if (isKeyDisallowed($key)) {
		$report['config'][$key] = 'xxxx Value is set but was removed by reporting app for security reasons xxxx';
		continue;
	}
	$report['config'][$key] = OC_Config::getValue($key);
}

// Get the phpinfo, parse it, and record it (parts from http://www.php.net/manual/en/function.phpinfo.php#87463)
ob_start();
phpinfo(-1);

$phpinfo = preg_replace(
	array('#^.*<body>(.*)</body>.*$#ms', '#<h2>PHP License</h2>.*$#ms',
		'#<h1>Configuration</h1>#', "#\r?\n#", "#</(h1|h2|h3|tr)>#", '# +<#',
		"#[ \t]+#", '#&nbsp;#', '#  +#', '# class=".*?"#', '%&#039;%',
		'#<tr>(?:.*?)" src="(?:.*?)=(.*?)" alt="PHP Logo" /></a>'
		. '<h1>PHP Version (.*?)</h1>(?:\n+?)</td></tr>#',
		'#<h1><a href="(?:.*?)\?=(.*?)">PHP Credits</a></h1>#',
		'#<tr>(?:.*?)" src="(?:.*?)=(.*?)"(?:.*?)Zend Engine (.*?),(?:.*?)</tr>#',
		"# +#", '#<tr>#', '#</tr>#'),
	array('$1', '', '', '', '</$1>' . "\n", '<', ' ', ' ', ' ', '', ' ',
		'<h2>PHP Configuration</h2>' . "\n" . '<tr><td>PHP Version</td><td>$2</td></tr>' .
		"\n" . '<tr><td>PHP Egg</td><td>$1</td></tr>',
		'<tr><td>PHP Credits Egg</td><td>$1</td></tr>',
		'<tr><td>Zend Engine</td><td>$2</td></tr>' . "\n" .
		'<tr><td>Zend Egg</td><td>$1</td></tr>', ' ', '%S%', '%E%'),
	ob_get_clean());

$sections = explode('<h2>', strip_tags($phpinfo, '<h2><th><td>'));
unset($sections[0]);

$report['phpinfo'] = array();
$sensitiveServerConfigs = array_flip($sensitiveServerConfigs);
foreach ($sections as $section) {
	$n = substr($section, 0, strpos($section, '</h2>'));
	if ($n === 'PHP Variables') {
		continue;
	}
	preg_match_all(
		'#%S%(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?%E%#',
		$section, $matches, PREG_SET_ORDER);
	foreach ($matches as $match) {
		if (isset($sensitiveServerConfigs[$match[1]])) {
			continue;
			// filter all key which contain 'password'
		}
		if(!isset($match[3])) {
			$value = isset($match[2]) ? $match[2] : null;
		}
		elseif($match[2] == $match[3]) {
			$value = $match[2];
		}
		else {
			$value = array_slice($match, 2);
		}
		$report['phpinfo'][$n][$match[1]] = $value;
	}
}

header('content-type: text/plain');
header('content-disposition: attachment; filename=config_report_' . date('Ymd') . '.txt');
$settings = defined('JSON_PRETTY_PRINT') ? JSON_PRETTY_PRINT : 0;
echo json_encode($report, $settings);
