/**
 * ownCloud
 *
 * @author Tom Needham <tom@owncloud.com>
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

(function ($, OC) {

    if (!OCA.Files_Drop) {
        OCA.Files_Drop = {};
    }

    var App = {

        uploadUrl: OC.generateUrl('/apps/files_drop/upload'),


        initialize: function() {
            $('.files_drop_container input[type="file"]').change(function (e) {
                e.stopPropagation();
                e.preventDefault();
                $('.files_drop_container .error').hide();
            });

            var fileUploadContainer = $('.files_drop_container');

            fileUploadContainer.bind('fileuploadsubmit', function (e, data) {
                var relativePath = data.files[0].relativePath;
                if (!relativePath) {
                    relativePath = '';
                }
                data.formData = {
                    token: fileUploadContainer.data('token'),
                    relativePath: relativePath,
                    requesttoken: oc_requesttoken
                };
            });

            fileUploadContainer.fileupload({
                url: OCA.Files_Drop.uploadUrl,
                dataType: 'json',
                maxFileSize: fileUploadContainer.data('maxupload'),
                messages: {
                    maxFileSize: t('files_drop', 'File exceeds maximum allowed size.')
                },
                progressall: function (e, data) {
                    var progressBar = $('#progressBar');
                    progressBar.removeClass('hidden');

                    var progress = parseInt(data.loaded / data.total * 100, 10);
                    $('#progressState').css('width', function () {
                        return progress + '%';
                    });
                    $('#progressPercent').text(progress + '%');

                    if(progress === 100) {
                        progressBar.addClass('hidden');
                    }
                }
            });

            fileUploadContainer
                .on('fileuploaddone', function (e, data) {
                    OCA.Files_Drop.addUploadedFile(data.files[0].name);
                })
                .on('fileuploadadd', function(evt, data) {
                    var $this = $(this);
                    var validation = data.process(function () {
                        return $this.fileupload('process', data);
                    });
                    validation.fail(function(data) {
                        OCA.Files_Drop.addErroredFile(data.files[0].name, data.files[0].error);
                    });
            });
        },

        /**
         * @param {string} uploadedFilename
         */
        addUploadedFile: function(uploadedFilename) {
            var uploadCompleteContainer =  $('.files_drop_container .completed');
            uploadCompleteContainer.removeClass('hidden');
            uploadCompleteContainer.children('ul').append('<li>' + escapeHTML(uploadedFilename) + '</li>');
        },

        /**
         * Add an error in case a file is too big
         * @param {string} uploadedFilename
         * @param {string} error
         */
        addErroredFile: function(uploadedFilename, error) {
            var errorDiv = $('#errorDiv');
            errorDiv.removeClass('hidden');

            errorDiv.children('ul').append('<li>' + escapeHTML(uploadedFilename) + ' (' + escapeHTML(error) + ')' + '</li>');
        }

    };

    OCA.Files_Drop = App;

    // Go go go
    $(document).ready(function() {
        OCA.Files_Drop.initialize();
    });

})(jQuery, OC);