<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\Lib;

use OCP\Files\NotFoundException;
use OCP\IConfig;
use OCP\Files\Folder;
use OCP\Files\Node;
use OCP\IURLGenerator;

/**
 * Class Endpoint describes the endpoint that gets stored in the users preferences
 *
 * @package OCA\Files_Drop\Lib
 */
class Endpoint {
	/** @var string */
	private $user;
	/** @var string */
	private $appName;
	/** @var string */
	private $token;
	/** @var int */
	private $fileId;
	/** @var Folder */
	private $userFolder;
	/** @var IURLGenerator */
	private $urlGenerator;
	/** @var HelperWrapper */
	private $helperWrapper;

	/**
	 * @param string $UserId
	 * @param string $AppName
	 * @param IConfig $config
	 * @param IURLGenerator $urlGenerator
	 * @param Node $userFolder
	 * @param HelperWrapper $helperWrapper
	 */
	public function __construct($UserId,
								$AppName,
								IConfig $config,
								IURLGenerator $urlGenerator,
								Node $userFolder,
								HelperWrapper $helperWrapper) {
		$this->user = $UserId;
		$this->userFolder = $userFolder;
		$this->appName = $AppName;
		$this->urlGenerator = $urlGenerator;
		$this->helperWrapper = $helperWrapper;
		$this->loadCurrentEndpoint($config);
	}

	/**
	 * @param IConfig $config
	 */
	private function loadCurrentEndpoint(IConfig $config) {
		$this->token = $config->getUserValue($this->user, $this->appName, 'token');
		$this->fileId = (int)$config->getUserValue($this->user, $this->appName, 'fileId');
	}

	/**
	 * @return string
	 */
	public function getUser() {
		return $this->user;
	}

	/**
	 * @return string
	 */
	public function getToken() {
		return $this->token;
	}

	/**
	 * @return string
	 */
	public function getUrl() {
		return $this->urlGenerator->linkToRouteAbsolute('files_drop.page.drop', ['token' => $this->getToken()]);
	}

	/**
	 * @param string $token
	 */
	public function setToken($token) {
		$this->token = $token;
	}

	/**
	 * @return int
	 */
	public function getFileId() {
		return $this->fileId;
	}

	/**
	 * @param int $fileId
	 */
	public function setFileId($fileId) {
		$this->fileId = $fileId;
	}

	/**
	 * @return Folder
	 * @throws NotFoundException
	 */
	public function getFolderView() {
		$this->helperWrapper->setupFS($this->getUser());

		$dropFolder = $this->userFolder->getById($this->getFileId());
		if(isset($dropFolder[0]) && $dropFolder[0] instanceof Folder) {
			return $dropFolder[0];
		}
		throw new NotFoundException();
	}

	/**
	 * @return string Relative path to the users' file directory
	 * @throws NotFoundException
	 */
	public function getRelativePath() {
		return $this->userFolder->getRelativePath($this->getFolderView()->getPath());
	}
}
