<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\tests\unit\lib;

use OCA\Files_Drop\Lib\Endpoint;
use Test\TestCase;
use OCP\IConfig;
use OCA\Files_Drop\Lib\HelperWrapper;
use OCP\IURLGenerator;
use OCP\Files\Folder;

/**
 * Class EndpointTest
 *
 * @package OCA\Files_Drop\tests\unit\lib
 */
class EndpointTest extends TestCase {
	/** @var Endpoint */
	private $endpoint;
	/** @var string */
	private $userId;
	/** @var string */
	private $appName;
	/** @var IConfig */
	private $config;
	/** @var IURLGenerator */
	private $urlGenerator;
	/** @var Folder */
	private $userFolder;
	/** @var HelperWrapper */
	private $helperWrapper;

	public function setUp() {
		parent::setUp();

		$this->userId = 'björn.schießle';
		$this->appName = 'files_drop';
		$this->config = $this->getMockBuilder('\OCP\IConfig')
			->disableOriginalConstructor()->getMock();
		$this->urlGenerator = $this->getMockBuilder('\OCP\IURLGenerator')
			->disableOriginalConstructor()->getMock();
		$this->userFolder = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();
		$this->helperWrapper = $this->getMockBuilder('\OCA\Files_Drop\Lib\HelperWrapper')
			->disableOriginalConstructor()->getMock();

		$this->config
			->expects($this->at(0))
			->method('getUserValue')
			->with($this->userId, $this->appName, 'token', '')
			->willReturn('TheToken');
		$this->config
			->expects($this->at(1))
			->method('getUserValue')
			->with($this->userId, $this->appName, 'fileId', '')
			->willReturn(1337);

		$this->endpoint = new Endpoint(
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		);
	}

	public function testGetUser() {
		$this->assertSame($this->userId, $this->endpoint->getUser());
	}

	public function testGetToken() {
		$this->assertSame('TheToken', $this->endpoint->getToken());
	}

	public function testGetUrl() {
		$this->urlGenerator
			->expects($this->once())
			->method('linkToRouteAbsolute')
			->with('files_drop.page.drop', ['token' => 'TheToken'])
			->willReturn('MyAbsoluteURL');
		$this->assertSame('MyAbsoluteURL', $this->endpoint->getUrl());
	}

	public function testSetToken() {
		$this->endpoint->setToken('Foo');
		$this->assertSame('Foo', $this->endpoint->getToken());
	}

	public function testGetFileId() {
		$this->assertSame(1337, $this->endpoint->getFileId());
	}

	public function testSetFileId() {
		$this->endpoint->setFileId(815);
		$this->assertSame(815, $this->endpoint->getFileId());
	}

	/**
	 * @expectedException \OCP\Files\NotFoundException
	 */
	public function testGetFolderViewException() {
		$this->userFolder
			->expects($this->once())
			->method('getById')
			->with(1337)
			->willReturn([]);
		$this->endpoint->getFolderView();
	}

	public function testGetFolderView() {
		$folder = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();

		$this->userFolder
			->expects($this->exactly(2))
			->method('getById')
			->with(1337)
			->willReturn([$folder]);
		$this->endpoint->getFolderView();
		$this->assertSame($folder, $this->endpoint->getFolderView());
	}

	/**
	 * @expectedException \OCP\Files\NotFoundException
	 */
	public function testGetRelativePathException() {
		$this->userFolder
			->expects($this->once())
			->method('getById')
			->with(1337)
			->willReturn([]);
		$this->endpoint->getRelativePath();
	}

	public function testGetRelativePath() {
		$folder = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();
		$folder
			->expects($this->once())
			->method('getPath')
			->willReturn('foo');

		$this->userFolder
			->expects($this->once())
			->method('getById')
			->with(1337)
			->willReturn([$folder]);
		$this->userFolder
			->expects($this->once())
			->method('getRelativePath')
			->with('foo')
			->willReturn('foo');

		$this->assertSame('foo', $this->endpoint->getRelativePath());
	}
}
