<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\WNDUtils;

OCP\JSON::checkAppEnabled('windows_network_drive');
OCP\JSON::checkLoggedIn();
OCP\JSON::callCheck();

$mandatoryParams = array('id', 'type');
$user = \OCP\User::getUser();
$groups = \OC_Group::getUserGroups($user);

$test = array();  // to hold the results

$l = OC_L10N::get('windows_network_drive');
// check mandatory parameters
foreach ($mandatoryParams as $mparam) {
    if (!isset($_GET[$mparam]) || ($_GET[$mparam] !== "0" && empty($_GET[$mparam]))) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Missing mandatory parameters'))));
        exit();
    }
}

// test can take sometime, so close the session as it won't be used
OC::$server->getSession()->close();

// get the mount
if ($_GET['type'] === 'admin') {
    $cursor = DBAccess::getMountByIdWithCreds(intval($_GET['id']), $user);
    if ($cursor === false || ($row = $cursor->fetchRow()) === false) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Mount point not found'))));
        exit();
    }

    // check if the global mount is available for the current user
    $cursor2 = DBAccess::getApplicablesForMountId($row['mount_id']);
    if ($cursor2 === false) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot get the applicable data'))));
        exit();
    }

    $isAvailableForUser = false;
    while (($row2 = $cursor2->fetchRow()) !== false) {
        if ($row2['mount_type'] === 'global' ||
            ($row2['mount_type'] === 'user' && $row2['mount_type_name'] === $user) ||
            ($row2['mount_type'] === 'group' && in_array($row2['mount_type_name'], $groups))) {
            $isAvailableForUser = true;
            break;
        }
    }

    if (!$isAvailableForUser) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Mount point not available for this user'))));
        exit();
    }

    WNDUtils::performTestOverGlobalRow($row, $test, $l);
    // output the data
    \OCP\JSON::success(array("data" => $test[$row['mount_point']]));

} else if ($_GET['type'] === 'personal') {
    $cursor = DBAccess::getPersonalMountByIdAndUser(intval($_GET['id']), $user);
    if ($cursor === false || ($row = $cursor->fetchRow()) === false) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Mount point not found for this user'))));
        exit();
    }

    WNDUtils::performTestOverPersonalRow($row, $test, $l);
    // output the data
    \OCP\JSON::success(array("data" => $test[$row['mount_point']]));
}

