<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::checkAdminUser();
\OCP\JSON::callCheck();

$l = OC_L10N::get('windows_network_drive');

if (!isset($_POST['WNDMountId'])) {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Missing data'))));
    exit();
}

$cursor = DBAccess::getMountByIdWithCreds($_POST['WNDMountId']);
if ($cursor !== false) {
    // clear the cache and storage data before removing the information from the DB
    // if the password is wrong (decrypting it returns false) we still try to clear the cache
    while (($row = $cursor->fetchRow()) !== false) {
        try {
            $wnd = \OCA\windows_network_drive\lib\WNDUtils::getWndForGlobalRow($row);
            if ($wnd) {
                $wnd->getCache()->clear();
            }
        } catch (\Exception $e) {
            // In case of a "Login Credentials" type, we might not be able to get the
            // WND connection, so we just ignore it, otherwise we rethrow the exception
            if (intval($row['use_custom_credentials']) !== 3) {
                throw $e;
            }
        }
    }
    DBAccess::deleteMountById($_POST['WNDMountId']);

    \OCP\JSON::success(array("data" => array("message" => $l->t('ok'))));
} else {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot get the mount point'))));
}

