<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\Utils;

OCP\JSON::checkLoggedIn();
OCP\JSON::callCheck();

if ($_SERVER['REQUEST_METHOD'] === 'GET') {
    $target = $_GET['target'];
    if (strpos($target, '..') !== false) {
        exit();
    }

    $id_username = Utils::getDefault($_GET, 'iduser', 'WNDuser');
    $id_password = Utils::getDefault($_GET, 'idpass', 'WNDpass');
    $id_mount = Utils::getDefault($_GET, 'idmount', 'WNDMountId');
    $actionUrl = OCP\Util::linkTo('windows_network_drive', 'ajax/' . $target);
    $mountPoint = $_GET["m"];
    $folder = $_GET['name'];
    $smburl = $_GET['url'];
    $smbshare = $_GET['share'];
    $extraInfo = $_GET['extra'];

    // list of expected parameters
    $fixedParams = array('m', 'name', 'url', 'share', 'extra', 'target', 'iduser',
                        'idpass', 'idmount');

    // we'll need a bit of special processing here: replace "\n\n" into "<br/>" and surround
    // the folder name with html tags
    $l = OC_L10N::get('windows_network_drive');
    $enterCredentialsText = $l->t('Please enter correct credentials to mount %s folder', '%s');
    $extra = str_replace("\n\n", '<br/>', OC_Util::sanitizeHTML($extraInfo));
    $sanitizedFolder = OC_Util::sanitizeHTML($folder);
    $translatedText = OC_Util::sanitizeHTML($enterCredentialsText);
    $spanText = str_replace('%s', "<strong>$sanitizedFolder</strong>", $translatedText);

    $tmpl = new OCP\Template('windows_network_drive', 'credentialsDialog');
    $tmpl->assign('spanText', $spanText);
    $tmpl->assign('service', $smburl);
    $tmpl->assign('share', $smbshare);
    $tmpl->assign('actionUrl', $actionUrl);
    $tmpl->assign('id_username', $id_username);
    $tmpl->assign('id_password', $id_password);
    $tmpl->assign('extra', $extra);

    // all "unexpected" parameters will be added as hidden values in the form
    $hiddenData = array();
    foreach ($_GET as $key => $value) {
        if (!in_array($key, $fixedParams, true)) {
            $hiddenData[$key] = $value;
        }
    }
    $hiddenData[$id_mount] = $mountPoint;

    $tmpl->assign('hiddenData', $hiddenData);
    OCP\JSON::success(array("form" => $tmpl->fetchPage()));
}
