<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::isLoggedIn();
\OCP\JSON::callCheck();

$l = OC_L10N::get('windows_network_drive');
if (!isset($_POST['WNDMountId'])) {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Missing mandatory parameters'))));
    exit();
}

$_POST['WNDuser'] = Utils::getDefault($_POST, 'WNDuser', '');
$_POST['WNDpass'] = Utils::getDefault($_POST, 'WNDpass', '');

$mountData = DBAccess::getMountById($_POST['WNDMountId']);
if ($mountData !== false) {
    if (($row = $mountData->fetchRow()) !== false) {
        $params = array('host' => $row['mount_url'],
                        'share' => $row['share'],
                        'root' => $row['root'],
                        'user' => $_POST['WNDuser'],
                        'password' => $_POST['WNDpass']);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            // perform a test to check if it possible to access, if the test is successful, save
            // the values
            if ($wnd->test()) {
                $mountingUser = \OCP\User::getUser();

                $cursor = DBAccess::getCustomCredentials($_POST['WNDMountId'], $mountingUser);
                if ($cursor !== false && ($customCreds = $cursor->fetchRow()) !== false) {
                    // check if we need to clear information
                    $decryptedPassword = Utils::decryptPassword($customCreds['credential_password']);
                    if ($decryptedPassword === false) {
						\OC_Log::write('wnd', 'Could not decrypt custom credentials for "' . $customCreds['credential_username'] . '" on "' . $row['mount_url'] . '"', \OC_Log::ERROR);
                    }
                    $oldParams = array('host' => $row['mount_url'],
                                    'share' => $row['share'],
                                    'root' => $row['root'],
                                    'user' => $customCreds['credential_username'],
                                    'password' => $decryptedPassword);
                    $oldwnd = new \OCA\windows_network_drive\lib\WND($oldParams);
                    if ($oldwnd->getId() !== $wnd->getId()) {
                        $oldwnd->getCache()->clear();
                    }
                }

                $encryptedPass = Utils::encryptPassword($_POST['WNDpass']);
                DBAccess::saveCustomCredentials($_POST['WNDMountId'], $mountingUser, 0, $_POST['WNDuser'], $encryptedPass);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
    } else {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Mount data not found'))));
        exit();
    }
} else {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Error accessing the DB'))));
    exit();
}


\OCP\JSON::success(array("data" => $l->t('ok')));
