<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::isLoggedIn();
\OCP\JSON::callCheck();

$mandatoryParams = array('WNDMountPoint', 'WNDUrl', 'WNDShare', 'WNDRoot', 'WNDCredentialsType');

// global username and password can be empty to connect as guest
$_POST['WNDuser'] = Utils::getDefault($_POST, 'WNDuser', '');
$_POST['WNDpass'] = Utils::getDefault($_POST, 'WNDpass', '');

$l = OC_L10N::get('windows_network_drive');
// check mandatory parameters
foreach ($mandatoryParams as $mparam) {
    if (!isset($_POST[$mparam]) || ($_POST[$mparam] !== "0" && empty($_POST[$mparam]))) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Missing mandatory parameters'))));
        exit();
    }
}

// check special chars in the mount point
Utils::checkInvalidChars($_POST['WNDMountPoint'], '\\/<>:"\'|?*', function($list) use ($l) {
    \OCP\JSON::error(array("data" => array("message" => $l->t('The mount point contains one of the following invalid chars: %s', implode(', ', $list)))));
    exit();
});

$personal_enabled = \OCP\Config::getAppValue('windows_network_drive', 'personal_mounts', false);
if (!$personal_enabled) {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Personal mounts are disabled'))));
    exit();
}

$user = \OCP\User::getUser();

switch ($_POST['WNDCredentialsType']) {
    case "0":
        $params = array('host' => $_POST['WNDUrl'],
                        'share' => $_POST['WNDShare'],
                        'root' => $_POST['WNDRoot'],
                        'user' => $_POST['WNDuser'],
                        'password' => $_POST['WNDpass']);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            if ($wnd->test()) {
                $encryptedPass = Utils::encryptPassword($_POST['WNDpass']);
                DBAccess::savePersonalMount($_POST['WNDMountPoint'], $_POST['WNDUrl'], $_POST['WNDShare'],
                                            $_POST['WNDRoot'], $user, 0, $_POST['WNDuser'],
                                            $encryptedPass);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
        break;

    case "2":
        $gcreds = DBAccess::getPersonalGlobalCredentialsCached($user);
        $decryptedPassword = Utils::decryptPassword($gcreds['global_password']);
        if ($decryptedPassword === false) {
			\OC_Log::write('wnd', 'Could not decrypt global personal credentials for "' . $gcreds['global_username'] . '"" on "' . $_POST['WNDUrl'] . '"', \OC_Log::ERROR);
        }
        $params = array('host' => $_POST['WNDUrl'],
                        'share' => $_POST['WNDShare'],
                        'root' => $_POST['WNDRoot'],
                        'user' => $gcreds['global_username'],
                        'password' => $decryptedPassword);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            if ($wnd->test()) {
                DBAccess::savePersonalMount($_POST['WNDMountPoint'], $_POST['WNDUrl'],
                                $_POST['WNDShare'], $_POST['WNDRoot'], $user, 2, null, null);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
        break;

}

\OCP\JSON::success(array("data" => $l->t('ok')));
