<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::isLoggedIn();
\OCP\JSON::callCheck();

$l = OC_L10N::get('windows_network_drive');
$mandatoryParams = array('WNDMountId', 'type');
foreach ($mandatoryParams as $mparam) {
    if (!isset($_POST[$mparam]) || ($_POST[$mparam] !== "0" && empty($_POST[$mparam]))) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Missing mandatory parameters'))));
        exit();
    }
}

$_POST['WNDuser'] = Utils::getDefault($_POST, 'WNDuser', '');
$_POST['WNDpass'] = Utils::getDefault($_POST, 'WNDpass', '');

$user = \OCP\User::getUser();

// if type = 0 we'll use the credentials passed as parameter, otherwise we'll use the personal
// credentials for the user
if (intval($_POST['type']) === 0) {
    $cuser = $_POST['WNDuser'];
    $cpass = $_POST['WNDpass'];
} else {
    $globalCredentials = DBAccess::getPersonalGlobalCredentialsCached($user);
    $cuser = $globalCredentials['global_username'];
    $cpass = Utils::decryptPassword($globalCredentials['global_password']);
    if ($cpass === false) {
		\OC_Log::write('wnd', 'Could not decrypt global personal credentials for "' . $cuser . '"" on Mount #' . $_POST['WNDMountId'], \OC_Log::ERROR);
    }
}

$cursor = DBAccess::getPersonalMountByIdAndUser($_POST['WNDMountId'], $user);
if ($cursor !== false) {
    // we'll only access to the first row (mount id should be unique anyway)
    if (($row = $cursor->fetchRow()) !== false) {
        $params = array('host' => $row['mount_url'],
                        'share' => $row['share'],
                        'root' => $row['root'],
                        'user' => $cuser,
                        'password' => $cpass);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            if ($wnd->test()) {
                // clear cache data of old wnd mount point (we could have duplicated info in the DB)
                $oldwnd = \OCA\windows_network_drive\lib\WNDUtils::getWndForPersonalRow($row);
                if ($oldwnd && ($wnd->getId() !== $oldwnd->getId())) {
                    $oldwnd->getCache()->clear();
                }

                $encryptedPass = Utils::encryptPassword($cpass);
                DBAccess::updatePersonalMountCredentialsType(intval($_POST['WNDMountId']), intval($_POST['type']), $cuser, $encryptedPass);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
    } else {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Mount point not found'))));
        exit();
    }
} else {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Error accessing data'))));
    exit();
}

\OCP\JSON::success(array("data" => $l->t('ok')));
