/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

function PersonalTableCodeGenerator () {
    this.generateTableRow = function(mount) {
        var raw = '<tr data-mid="' + escapeHTML(mount.mount_id).replace(/"/g, "&quot;") + '">';
        raw += '<td>' + escapeHTML(mount.mount_point) + '</td>';
        raw += '<td>' + escapeHTML(mount.mount_url) + '</td>';
        raw += '<td>' + escapeHTML(mount.share) + '</td>';
        raw += '<td>' + escapeHTML(mount.root) + '</td>';
        raw += '<td class="td_wnd_creds">';
        if (parseInt(mount.use_custom_credentials) === 1) {
            if (mount.credential_username === null || mount.credential_password === null) {
                raw += '<input type="text" id="WNDuser' + escapeHTML(mount.mount_id).replace(/"/g, "&quot;") + '" name="WNDuser" data-copyfromid="WNDGlobalUsername" placeholder="' + escapeHTML(t('windows_network_drive', 'Username')).replace(/"/g, "&quot;") + '" />';
                raw += '<input type="password" id="WNDpass' + escapeHTML(mount.mount_id).replace(/"/g, "&quot;") + '" name="WNDpass" data-copyfromid="WNDGlobalPassword" placeholder="' + escapeHTML(t('windows_network_drive', 'Password')).replace(/"/g, "&quot;") + '" />';
            } else {
                raw += '<span>' + escapeHTML(t('windows_network_drive', 'Username')) + ': </span><span name="username">' + escapeHTML(mount.credential_username) + '</span>';
            }
        } else {
            raw += '<span>' + escapeHTML(t('windows_network_drive', 'Credentials provided by the admin')) + '</span>';
        }
        raw += '</td>';

        raw += '<td>';
        if (parseInt(mount.use_custom_credentials) === 1) {
            if (mount.credential_username === null || mount.credential_password === null) {
                raw += '<input type="button" class="SaveButton" value="' + escapeHTML(t('windows_network_drive', 'Save')).replace(/"/g, "&quot;") + '" />';
            } else {
                raw += '<input type="button" class="EditButton" value="' + escapeHTML(t('windows_network_drive', 'Edit')).replace(/"/g, "&quot;") + '" />';
            }
        }
        raw += '</td>';
        return raw;
    };

    this.editRow = function(trrow) {
        var currentUsername = trrow.find('.td_wnd_creds span[name="username"]').text();
        var usernameInput = '<input type="text" name="guser" value="' + escapeHTML(currentUsername).replace(/"/g, "&quot;") + '" />';
        var passwordInput = '<input type="password" name="gpass" placeholder="' + escapeHTML(t('windows_network_drive', 'Password')).replace(/"/g, "&quot;") + '"/>';
        trrow.find('.td_wnd_creds').empty()
                                    .append(usernameInput + passwordInput)
                                    .find('input[type="button"]')
                                    .click(function(){
                                            trrow.find('.td_wnd_creds input[name="guser"]')
                                                .val($('#WNDGlobalUsername').val());
                                            trrow.find('.td_wnd_creds input[name="gpass"]')
                                                .val($('#WNDGlobalPassword').val());
        });
    };
}

function PersonalTableManager(tbody) {
    this.tbody = tbody;
    this.wndAjaxCalls = new WndAjaxCalls();
    this.codeGenerator = new PersonalTableCodeGenerator();

    this.attachEditHandlers = function() {
        var codeGen = this.codeGenerator;
        var wnd = this.wndAjaxCalls;
        var currentTableManager = this;
        this.tbody.find('input.EditButton[type="button"]').on('click.edit', function editClick(){
            var trrow = $(this).parent().parent();
            var tdCreds = trrow.find('.td_wnd_creds');
            tdCreds.data('oldCredInfo', tdCreds.html());
            var eventId = Math.random().toString().slice(2);
            $(document).on('click.wndoutside.' + eventId, function(e){
                if (!$(e.target).closest(trrow).length) {
                    tdCreds.html(tdCreds.data('oldCredInfo'));
                    trrow.find('input.EditButton[type="button"]').val(t('windows_network_drive', 'Edit'))
                                                                    .off('click.update')
                                                                    .on('click.edit', editClick);
                    $(document).off('click.wndoutside.' + eventId);
                }
            });
            codeGen.editRow(trrow);
            $(this).off('click.edit')
                    .val(t('windows_network_drive', 'Update'))
                    .on('click.update', function(){
                            var user = trrow.find('input[name="guser"]').val();
                            var pwd = trrow.find('input[name="gpass"]').val();
                            wnd.saveUserCredentials(trrow.data('mid'), user, pwd, function(response){
                                    if (response.status === "success") {
                                        currentTableManager.reloadTable();
                                    } else {
                                        if (response.data && response.data.message) {
                                            OC.dialogs.alert(response.data.message,
                                                    escapeHTML(t('windows_network_drive', 'Error checking Windows network drive configuration')));
                                        } else {
                                            OC.dialogs.alert(escapeHTML(t('windows_network_drive', 'Internal server error')),
                                                        escapeHTML(t('windows_network_drive', 'Error checking Windows network drive configuration')));
                                        }
                                    }
                            }, function(jqhrx, errorStatus, errorMessage){
                                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                                          escapeHTML(t('windows_network_drive', 'Internal error')));

                            });
                     });
        });
    };

    this.attachSaveHandlers = function() {
        var wnd = this.wndAjaxCalls;
        var currentTableManager = this;
        this.tbody.find('input.SaveButton[type="button"]').on('click.save', function(){
            currentTableManager.saveData($(this).parent().parent());
        });
    };

    this.reloadTable = function() {
        var codeGen = this.codeGenerator;
        var targetBody = this.tbody;
        var currentTableManager = this;
        this.wndAjaxCalls.getMountsForUser('admin', function(response){
            if (response.status === 'success') {
                var allRows = '';
                $.each(response.data, function(index, mount){
                    allRows += codeGen.generateTableRow(mount);
                });
                targetBody.empty().append(allRows);

                currentTableManager.attachEditHandlers();
                currentTableManager.attachSaveHandlers();
            }
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    };

    this.saveData = function(baseElement) {
        var allData = {};

        baseElement.find("input, select").each(function(){
            var thisElement = $(this);
            if (thisElement.is("select")) {
                var values = thisElement.val() || [];
                allData[thisElement.attr("name")] = values.join(",");
            } else if (thisElement.attr("type") === "text" || thisElement.attr("type") === "password") {
                allData[thisElement.attr("name")] = thisElement.val();
            } else if (thisElement.attr("type") === "checkbox") {
                allData[thisElement.attr("name")] = thisElement.prop("checked");
            }
        });

        var currentTableManager = this;
        this.wndAjaxCalls.saveUserCredentials(baseElement.data('mid'), allData.WNDuser, allData.WNDpass, function(response){
            if (response.status === "success") {
                // currentTableManager.extraClearInputs(baseElement);
                currentTableManager.reloadTable();
            } else {
                if (response.data && response.data.message) {
                    OC.dialogs.alert(response.data.message,
                            escapeHTML(t('windows_network_drive', 'Error checking Windows Network Drive Configuration')));
                } else {
                    OC.dialogs.alert(escapeHTML(t('windows_network_drive', 'Internal server error')),
                                escapeHTML(t('windows_network_drive', 'Error checking Windows Network Drive Configuration')));
                }

            }
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    };
}

function PersonalMountTableCodeGenerator () {
    this.generateTableRow = function(mount) {
        var raw = '<tr data-mid="' + escapeHTML(mount.personal_mount_id).replace(/"/g, "&quot;") + '">';
        raw += '<td>' + escapeHTML(mount.mount_point) + '</td>';
        raw += '<td>' + escapeHTML(mount.mount_url) + '</td>';
        raw += '<td>' + escapeHTML(mount.share) + '</td>';
        raw += '<td>' + escapeHTML(mount.root) + '</td>';
        raw += '<td class="td_wnd_creds">' + this.generateCredentialsTd(mount) + '</td>';
        raw += '<td>';
            raw += '<input type="button" class="EditButton" value="' + escapeHTML(t('windows_network_drive', 'Edit')).replace(/"/g, "&quot;") + '" /> ';
            raw += '<input type="button" class="DeleteButton" value="' + escapeHTML(t('windows_network_drive', 'Delete')).replace(/"/g, "&quot;") + '" />';
        raw += '</td>';
        return raw;
    };

    this.generateCredentialsTd = function(mount) {
        var credType = parseInt(mount.use_global_credentials);
        var raw = '<select class="wnd-select" disabled="disabled">';
        raw += '<option value="2"' + ((credType === 2) ? ' selected="selected"' : '') + '>' + escapeHTML(t('windows_network_drive', 'Personal credentials')) + '</option>';
        raw += '<option value="0"' + ((credType === 0) ? ' selected="selected"' : '') + '>' + escapeHTML(t('windows_network_drive', 'Custom credentials')) + '</option>';
        raw += '</select> ';

        if (credType !== 0) {
            raw += '<input type="text" name="WNDuser" placeholder="' + escapeHTML(t('windows_network_drive', 'Username')) + '" style="display:none" /> ';
            raw += '<input type="password" name="WNDpass" placeholder="' + escapeHTML(t('windows_network_drive', 'Password')) + '" style="display:none" />';
        } else {
            raw += '<input type="text" name="WNDGuser" placeholder="' + escapeHTML(t('windows_network_drive', 'Username')) + '" value="' + escapeHTML(mount.creds_username) + '" disabled="disabled" /> ';
            raw += '<input type="password" name="WNDGpass" placeholder="' + escapeHTML(t('windows_network_drive', 'Password')) + '" disabled="disabled" />';
        }
        return raw;
    };

    this.editRow = function(trrow) {
        trrow.find('.td_wnd_creds').find('select, input').prop('disabled', false);
        trrow.find('.td_wnd_creds select.wnd-select').change();
    };
}

function PersonalMountTableManager(tbody) {
    this.tbody = tbody;
    this.wndAjaxCalls = new WndAjaxCalls();
    this.codeGenerator = new PersonalMountTableCodeGenerator();

    this.attachEditHandlers = function() {
        var codeGen = this.codeGenerator;
        var wnd = this.wndAjaxCalls;
        var currentTableManager = this;
        this.tbody.find('input.EditButton[type="button"]').on('click.edit', function editClick(){
            var trrow = $(this).parent().parent();
            var tdCreds = trrow.find('.td_wnd_creds');
            tdCreds.data('oldCredInfo', tdCreds.html());
            tdCreds.find('select.wnd-select').change(function(){
                // the handler will disapear if clicked outside or updated
                var thisElement = $(this);
                if (thisElement.val() === "0") {
                    thisElement.siblings('input[name="WNDuser"],input[name="WNDpass"]').show();
                } else {
                    thisElement.siblings('input[name="WNDuser"],input[name="WNDpass"]').hide();
                }

            });
            var eventId = Math.random().toString().slice(2);
            $(document).on('click.wndoutside.' + eventId, function(e){
                if (!$(e.target).closest(trrow).length) {
                    // reset the chosen because js handlers won't be in place
                    tdCreds.find('select.wnd-select').select2('destroy');
                    tdCreds.html(tdCreds.data('oldCredInfo'));

                    tdCreds.find('div.wnd-select').remove();
                    tdCreds.find('select.wnd-select').select2({minimumResultsForSearch: -1, width: '160px'});

                    trrow.find('input.EditButton[type="button"]').val(t('windows_network_drive', 'Edit'))
                                                                    .off('click.update')
                                                                    .on('click.edit', editClick);
                    $(document).off('click.wndoutside.' + eventId);
                }
            });
            codeGen.editRow(trrow);
            $(this).off('click.edit')
                    .val(t('windows_network_drive', 'Update'))
                    .on('click.update', function(){
                            var user = trrow.find('input[name="WNDuser"]').val();
                            var pwd = trrow.find('input[name="WNDpass"]').val();
                            var type = trrow.find('td.td_wnd_creds select').val();
                            wnd.updatePersonalMountWithType(trrow.data('mid'), type, user, pwd, function(response){
                                    if (response.status === "success") {
                                        currentTableManager.reloadTable();
                                    } else {
                                        if (response.data && response.data.message) {
                                            OC.dialogs.alert(response.data.message,
                                                    escapeHTML(t('windows_network_drive', 'Error checking Windows Network Drive Configuration')));
                                        } else {
                                            OC.dialogs.alert(escapeHTML(t('windows_network_drive', 'Internal server error')),
                                                        escapeHTML(t('windows_network_drive', 'Error checking Windows Network Drive Configuration')));
                                        }
                                    }
                            }, function(jqhrx, errorStatus, errorMessage){
                                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                                          escapeHTML(t('windows_network_drive', 'Internal error')));

                            });
                     });
        });
    };

    this.attachDeleteHandlers = function() {
        var wnd = this.wndAjaxCalls;
        this.tbody.find('input.DeleteButton[type="button"]').click(function(){
            var trrow = $(this).parent().parent();
            wnd.deletePersonalMount(trrow.data('mid'), function(response){
                if (response.status === 'success') {
                    trrow.remove();
                }
            }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

            });
        });
    };

    this.reloadTable = function() {
        var codeGen = this.codeGenerator;
        var targetBody = this.tbody;
        var currentTableManager = this;
        this.wndAjaxCalls.getMountsForUser('personal', function(response){
            if (response.status === 'success') {
                var allRows = '';
                $.each(response.data, function(index, mount){
                    allRows += codeGen.generateTableRow(mount);
                });
                targetBody.empty().append(allRows);
                $(document).off('click.wndoutside');

                currentTableManager.attachEditHandlers();
                currentTableManager.attachDeleteHandlers();
                targetBody.find("select.wnd-select").select2({minimumResultsForSearch: -1, width: '160px'});
            }
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    };

    this.extraClearInputs = function(baseElement) {
        baseElement.find("input, select").each(function(){
            var thisElement = $(this);
            if (thisElement.is("select")) {
                thisElement.find('option').prop("selected", false)
                                    .filter('[value="global"],[selected]')
                                    .prop("selected", true);
                thisElement.trigger("liszt:updated").change();
            } else if (thisElement.attr("type") === "text" || thisElement.attr("type") === "password") {
                thisElement.val('');
            } else if (thisElement.attr("type") === "checkbox") {
                thisElement.prop("checked", false);
                thisElement.change();
            }
        });
    };

    this.saveData = function(baseElement) {
        var allData = {};

        baseElement.find("input, select").each(function(){
            var thisElement = $(this);
            if (thisElement.is("select[multiple]")) {
                var values = thisElement.val() || [];
                allData[thisElement.attr("name")] = values.join(",");
            } else if (thisElement.attr("type") === "text" || thisElement.attr("type") === "password" || thisElement.is("select:not([multiple])")) {
                allData[thisElement.attr("name")] = thisElement.val();
            } else if (thisElement.attr("type") === "checkbox") {
                allData[thisElement.attr("name")] = thisElement.prop("checked");
            }
        });

        var currentTableManager = this;
        this.wndAjaxCalls.savePersonalMount(allData, function(response){
            if (response.status === "success") {
                currentTableManager.extraClearInputs(baseElement);
                currentTableManager.reloadTable();
            } else {
                if (response.data && response.data.message) {
                    OC.dialogs.alert(response.data.message,
                            escapeHTML(t('windows_network_drive', 'Error checking Windows Network Drive Configuration')));
                } else {
                    OC.dialogs.alert(escapeHTML(t('windows_network_drive', 'Internal server error')),
                                escapeHTML(t('windows_network_drive', 'Error checking Windows Network Drive Configuration')));
                }
            }
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    };
}

$(document).ready(function(){
    var tableManager = new PersonalTableManager($('#WNDadminPersonalMountPoints table tbody'));

    tableManager.attachSaveHandlers();
    tableManager.attachEditHandlers();

    var personalMountTableManager = new PersonalMountTableManager($('#WNDPersonalMountPoints table tbody'));
    personalMountTableManager.attachEditHandlers();
    personalMountTableManager.attachDeleteHandlers();

    $('select.wnd-select').select2({minimumResultsForSearch: -1, width: '160px'});

    var isEmpty = function (element){
        value = $.trim(element.val());
        return (value === null || value === '');
    };

    $("#WNDNewMountForm input, #WNDNewMountForm select, #WNDpersonal [name=WNDGlobalUsername], #WNDpersonal [name=WNDGlobalPassword]").on("change input", function() {

        var elements = ["#WNDNewMountForm [name=WNDMountPoint]",
                        "#WNDNewMountForm [name=WNDUrl]",
                        "#WNDNewMountForm [name=WNDShare]",
                        "#WNDNewMountForm [name=WNDRoot]"];
        var credentialsType = $("#WNDNewMountForm [name=WNDCredentialsType]").val();

        if(credentialsType === "2") {
            elements.push("#WNDpersonal [name=WNDGlobalUsername]", "#WNDpersonal [name=WNDGlobalPassword]");
        }
        else if(credentialsType === "0"){
            elements.push("#WNDNewMountForm [name=WNDuser]", "#WNDNewMountForm [name=WNDpass]");
        }
        var validate = true;
        $.each(elements, function(index, element) {
            if (isEmpty($(element))){
               $("#WNDNewMountForm [nam=WNDSaveButton]").attr("disabled", "disabled");
               validate = false;
               //Stop each
               return validate;
            }
        });
        if(validate) $("#WNDNewMountForm [name=WNDSaveButton]").removeAttr('disabled');
    });


    $('#WNDSaveButton').click(function() {
        personalMountTableManager.saveData($(this).parent().parent());
        // base element should be the tfoot's tr
    });

    var wndTypingTimer = {WNDGlobalUsername : null, WNDGlobalPassword : null};
    $('#WNDGlobalUsername, #WNDGlobalPassword').on('keyup input', function(){
        var _self = $(this);
        var _self_id = _self.attr('id');
        var map = {WNDGlobalUsername : 'WNDuser', WNDGlobalPassword : 'WNDpass'};

        clearTimeout(wndTypingTimer[_self_id]);
        wndTypingTimer[_self_id] = setTimeout(function() {

            var dataToSend = {type: 'personal'};
            dataToSend[map[_self_id]] = _self.val();

            $.post(OC.filePath('windows_network_drive', 'ajax', 'saveGlobalCredentials.php'),
                dataToSend,
                function(response){
                    if (response.status === 'success') {
                        OCA.WND.WNDUtils.showAlert(t('windows_network_drive', 'Credentials stored'));
                    } else {
                        OC.dialogs.message(t('windows_network_drive', 'Something went wrong saving the credentials'), t('windows_network_drive', 'Failure'));
                    }
                });
            }, 750);
    }).on('click', function(){
        $(this).val('').trigger('input');
    });

    $('#WNDCredentialsType').change(function(){
        var thisElement = $(this);
        if (thisElement.val() === "0") {
            thisElement.siblings('#WNDuser, #WNDpass').show();
        } else {
            thisElement.siblings('#WNDuser, #WNDpass').hide();
        }
    });

    var preajaxCallback = function(positionElement) {
        if (typeof this.showTimes === "undefined") {
            this.showTimes = 0;
        }
        if (this.showTimes > 0) {
            this.showTimes++;
            return;
        }

        var wndBaseDiv = $('#WNDpersonal');
        if (wndBaseDiv.children('img.loading').length === 0) {
            wndBaseDiv.append('<img class="loading" style="display:none ; position:absolute" src="' + OC.imagePath('core', 'loading.gif') + '" />');
        }
        var loadingGif = $('#WNDpersonal > img.loading');
        loadingGif.show().position({my: 'center center',
                            at: 'center center',
                            of: positionElement});
        this.showTimes++;
    };

    var postajaxCallback = function() {
        if (this.showTimes > 1) {
            this.showTimes--;
            return;
        }
        $('#WNDpersonal > img.loading').hide();
        this.showTimes--;
    };

    tableManager.wndAjaxCalls.preajaxCallback = function() { preajaxCallback.call(tableManager.wndAjaxCalls, '#WNDadminPersonalMountPoints table.grid');};
    tableManager.wndAjaxCalls.postajaxCallback = postajaxCallback;
    personalMountTableManager.wndAjaxCalls.preajaxCallback = function() { preajaxCallback.call(personalMountTableManager.wndAjaxCalls, '#WNDPersonalMountPoints table.grid');};
    personalMountTableManager.wndAjaxCalls.postajaxCallback = postajaxCallback;
});
