<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\windows_network_drive\lib;

/**
 * A general utilities class for the windows network drive app
 */
class Utils {
    /**
     * Encrypt the string
     *
     * @param string $password text to be encryted
     * @return string the encrypted text
     */
    public static function encryptPassword($password) {
        $cipher = new \Crypt_Rijndael(CRYPT_RIJNDAEL_MODE_CBC);
        $iv = \OC_Util::generateRandomBytes(16);
        $cipher->setIV($iv);
        $cipher->setKey(\OCP\Config::getSystemValue('passwordsalt'));
        $encryptedPassword = base64_encode($iv . $cipher->encrypt($password));
        return $encryptedPassword;
    }

    /**
     * Decrypt the string
     *
     * @param string $storedPassword text to be decryted
     * @return string|false the decrypted text or false if the storedPassword is wrong
     */
    public static function decryptPassword($storedPassword) {
        $cipher = new \Crypt_Rijndael(CRYPT_RIJNDAEL_MODE_CBC);
        $encryptedPassword = base64_decode($storedPassword);
        $iv = substr($encryptedPassword, 0, 16);
        $binaryPassword = substr($encryptedPassword, 16);
        if ($binaryPassword === false) {
            return false;
        }
        $cipher->setIV($iv);
        $cipher->setKey(\OCP\Config::getSystemValue('passwordsalt'));
        $password = $cipher->decrypt($binaryPassword);
        return $password;
    }

    /**
     * Check for invalid char in the string and perform an action if needed
     *
     * @param string $stringToCheck the string to be checked
     * @param string $charList a string with the invalid chars
     * @param callable $action the action to be taken if there are any invalid chars
     * To the action an array with the single-qouted chars will be passed as parameter
     */
    public static function checkInvalidChars($stringToCheck, $charList, $action) {
        if (strpbrk($stringToCheck, $charList) !== false) {
            $invalidCharsList = array_map(function($e){ return '\'' . $e . '\''; }, array_unique(str_split($charList, 1)));
            call_user_func($action, $invalidCharsList);
        }
    }

    /**
     * Get the value for the key in the array or the default value if the key is not set.
     * This function will check with the isset() function to check if the key is in the array,
     * so notice that there might be special cases where the function will get the default
     * value instead of the real one
     *
     * @param array $arr the array where the key will be looked for
     * @param mixed $key the key for the array
     * @param mixed $default the value that will be returned if the check fails
     * @return mixed the value contained in the array or the default passed
     */
    public static function getDefault($arr, $key, $default = null) {
        return (isset($arr[$key])) ? $arr[$key] : $default;
    }
}
