/*
 * Copyright (c) 2014
 *
 * This file is licensed under the Affero General Public License version 3
 * or later.
 *
 * See the COPYING-README file.
 *
 */

/* global module, protractor, element, by, browser, require */
(function() {
  var params = browser.params;

	var Page = function() {
	};
  
  // Login locators 
  Page.loginForm = element(by.name('login'));
  Page.userInput = Page.loginForm.element(by.id('user'));
  Page.passwordInput = Page.loginForm.element(by.id('password'));
  Page.loginButton = element(by.id('submit'));
  Page.expandDisplayName = element(by.id('expandDisplayName'));

  Page.login = function(user, pass) {
    // logout 
    browser.manage().deleteAllCookies(); 
    
    return browser.get(params.baseUrl).then(function() {
      Page.userInput.sendKeys(user);
      Page.passwordInput.sendKeys(pass);
      Page.loginButton.click();

      var button = Page.expandDisplayName;
      return browser.wait(function() {
        return button.isPresent();
      }, params.wait, 'login wait expired - try to raise timeout with params.wait ');
    });
  };

	Page.logout = function() {
		browser.manage().deleteAllCookies(); 
    return browser.get(params.baseUrl);
	};

	//================ LOCATOR FUNCTIONS ====================================//
	Page.appsMenuId = function() {
		return by.css('#header .menutoggle');
	};

	Page.appMenuEntryId = function(appId) {
		return by.css('nav #apps [data-id=\'' + appId + '\']');
	};

	Page.settingsMenuId = function() {
		return by.css('#header #settings');
	};

  // topbar
  Page.displayNameId = function() {
   return by.id("expandDisplayName");
  } 
  
  Page.userActionDropdownId = function() {
   return by.id('expanddiv');
  }  

//================ PAGE NAVIGATION =====================================================//
//======================================================================================//

  /**
  * open / close upper left app menu
  */

  Page.toggleAppsMenu = function() {
  	var el = element(this.appsMenuId());
  	return el.click();
  };
  
  /**
  * checks if user is logged in, 
  * if yes, returns name of user is currently logged in 
  * if not, returns false,
  *
  */

  Page.isLoggedIn = function() {
    var displayName = Page.expandDisplayName;

    return displayName.isPresent().then(function(loggedIn) {
      if(loggedIn) {
        return displayName.getText().then(function(text){
          return text
        });
      } else {
        return false
      }
    });
  }

  /**
  * get a given url after logging in with given user / pass 
  *
  * @param {String} user username
  * @param {String} pass user's password
  * @param {String} [url] default filespage / page's url to get 
  */

  Page.getAsUser = function(user, pass, url) { 
    if(url == undefined) {
      url = params.baseUrl;
    };

    return Page.isLoggedIn().then(function(loggedIn) {
      if(loggedIn == user) {
        return browser.get(url);
      };
      return Page.login(user, pass).then(function() {
        return browser.get(url);
      });
    });
  };

//================ MOUSE ACTIONS =======================================================//
//======================================================================================//

  /**
  * mouves mouse to a specific page element
  *
  * @param {String} locator element's locator mouse is mouved to
  */

  Page.moveMouseTo = function(locator) {
    var elem = element(locator);
    browser.actions().mouseMove(elem).perform();
    return browser.wait(function() {
      // works just for td elements at the moment
      return element(by.css('td:hover')).isPresent();
    });
  };

  /**
  * drags and drops an page element on an other element 
  *
  * @param {String} locator1 element's locator gets draged
  * @param {String} locator2 element's locator gets droped on
  */

  Page.dragAndDrop = function (locator1, locator2) {
    this.moveMouseTo(locator1);
    browser.actions().mouseDown().perform();
    // can't use mouveMouseTo function cause element the draged element get's droped on don't hover
    var elem = element(locator2);
    browser.actions().mouseMove(elem).perform(); 
    browser.actions().mouseUp().perform();
  };

	module.exports = Page;
})();
