/*
 * Copyright (c) 2014
 *
 * This file is licensed under the Affero General Public License version 3
 * or later.
 *
 */

var Page = require('./helper/page.js');
var WindowsDrivePage = require('./pages/windowsDrive.page.js');
var FilesPage = require('./pages/files.page.js');

describe('Window Network Drive App', function() {
	var params = browser.params;
	var wndPage,
    filesPage;

	beforeEach(function() {
		isAngularSite(false);
    wndPage = new WindowsDrivePage(params.baseUrl);
    filesPage = new FilesPage(params.baseUrl);
    wndPage.getAsUser(params.login.user, params.login.password);
	});

	afterEach(function() {
    Page.logout();
	});
  
  it('connects with global credentials', function() {
    wndPage.globalUserName(params.wnd.user);
    wndPage.globalPassword(params.wnd.password);
    wndPage.mountPointInput.sendKeys('test_cifs-global');
    wndPage.availableForInput.sendKeys('Admin');
    wndPage.availableForInput.sendKeys(protractor.Key.ENTER);
    wndPage.urlInput.sendKeys(params.wnd.url);
    wndPage.shareInput.sendKeys(params.wnd.share);
    wndPage.rootInput.sendKeys(params.wnd.root);
    wndPage.selectCredentials(1);
            
    wndPage.save();
    expect(wndPage.ocDialogTitle().isPresent()).toBeFalsy();
    expect(wndPage.mountPoints.count()).toEqual(1);

    filesPage.get();
    expect(filesPage.listFiles()).toContain('test_cifs-global');

    // Cleanup
    wndPage.get();
    wndPage.deleteButton.click();
    wndPage.globalUserName('');
    wndPage.globalPassword('');
    
    filesPage.get();
    expect(filesPage.listFiles()).not.toContain('test_cifs-global');
  });
  
  it('connects with directly given credentials', function() {
    var mountPoint = 'test_cifs-direct';
    wndPage.mountPointInput.sendKeys(mountPoint);
    wndPage.availableForInput.sendKeys('Admin');
    wndPage.availableForInput.sendKeys(protractor.Key.ENTER);
    wndPage.urlInput.sendKeys(params.wnd.url);
    wndPage.shareInput.sendKeys(params.wnd.share);
    wndPage.rootInput.sendKeys(params.wnd.root);
    wndPage.selectCredentials(2);
    wndPage.gUserInput.sendKeys(params.wnd.user);
    wndPage.gPasswordInput.sendKeys(params.wnd.password);
    
    wndPage.save();
    expect(wndPage.ocDialogTitle().isPresent()).toBeFalsy();
    expect(wndPage.mountPoints.count()).toEqual(1);

    filesPage.get();
    expect(filesPage.listFiles()).toContain(mountPoint);

    // Cleanup
    wndPage.get();
    wndPage.deleteButton.click();
    
    filesPage.get();
    expect(filesPage.listFiles()).not.toContain(mountPoint);
  });
  
});
