<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\windows_network_drive\tests;

use \OCA\windows_network_drive\lib\DBAccess;

class dbAccessTest extends \PHPUnit_Framework_TestCase {
    public function test_saveGlobalMount() {
        $applicables = array('global' => true,
                            'groups' => array('ggg1', 'ggg2', 'ggg3'),
                            'users' => array('uuu1', 'uuu2', 'uuu3'));
        DBAccess::saveMount('ppp1', $applicables, 'localhost', 'home', '/', 0, 'ut1', 'ut2');
        $isSaved = false;
        $mountid = null;
        $cursor = DBAccess::getAllMounts();
        if ($cursor !== false) {
            while (($row = $cursor->fetchRow()) !== false) {
                if ($row['mount_point'] === 'ppp1' &&
                        $row['mount_url'] === 'localhost' &&
                        $row['share'] === 'home' &&
                        $row['root'] === '/' &&
                        intval($row['use_custom_credentials']) === 0 &&
                        $row['global_username'] === 'ut1' &&
                        $row['global_password'] === 'ut2') {

                    $isSaved = true;
                    $mountid = $row['mount_id'];
                    break;
                }
            }
        }
        $this->assertTrue($isSaved);

        $cursor2 = DBAccess::getApplicablesForMountId($mountid);
        $data = array();
        if ($cursor2 !== false) {
            while (($row2 = $cursor2->fetchRow()) !== false) {
                if (!isset($data[$row2['mount_type']])) {
                    $data[$row2['mount_type']] = array();
                }
                array_push($data[$row2['mount_type']], $row2['mount_type_name']);
            }
        }
        $this->assertArrayHasKey('global', $data);
        $this->assertSame(array_diff($applicables['groups'], $data['group']), array_diff($data['group'], $applicables['groups']));
        $this->assertSame(array_diff($applicables['users'], $data['user']), array_diff($data['user'], $applicables['users']));

        $expected = array('mount_id' => $mountid,
                                    'mount_point' => 'ppp1',
                                    'mount_url' => 'localhost',
                                    'share' => 'home',
                                    'root' => '/',
                                    'use_custom_credentials' => 0,
                                    'global_username' => 'ut1',
                                    'global_password' => 'ut2');
        $cursor = DBAccess::getMountById($mountid);
        $row = $cursor->fetchRow();
        $row['use_custom_credentials'] = intval($row['use_custom_credentials']);
        $this->assertEquals($expected, $row);
    }

    public function test_getMountForUser() {
        // check no dups
        $cursor = DBAccess::getMountsForUser('uuu1', array('ggg1', 'ggg2'));
        $rows = array();
        if ($cursor !== false) {
            while (($row = $cursor->fetchRow()) !== false) {
                if ($row['mount_point'] === 'ppp1') {
                    $rows[] = $row;
                }
            }
        }
        $this->assertEquals(count($rows), 1);
    }

    public function test_deleteMountPoint() {
        $cursor = DBAccess::getMountByProps('ppp1', 'localhost', 'home', '/');
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        DBAccess::deleteMountById($row['mount_id']);

        $cursor = DBAccess::getMountByProps('ppp1', 'localhost', 'home', '/');
        if ($cursor !== false) {
            $this->assertFalse($cursor->fetchRow());
        }
    }

    public function test_updateMountCreds() {
        $applicables = array('global' => true);
        DBAccess::saveMount('qazwsx1', $applicables, 'localhost', 'home', '/', 0, 'ut11', 'ut22');
        $cursor = DBAccess::getMountByProps('qazwsx1', 'localhost', 'home', '/');
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertEquals('ut11', $row['global_username']);
        $this->assertEquals('ut22', $row['global_password']);

        DBAccess::updateMountGlobalCreds(intval($row['mount_id']), 'mod11', 'mod22');

        $cursor = DBAccess::getMountByProps('qazwsx1', 'localhost', 'home', '/');
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertNotEquals('ut11', $row['global_username']);
        $this->assertNotEquals('ut22', $row['global_password']);
        $this->assertEquals('mod11', $row['global_username']);
        $this->assertEquals('mod22', $row['global_password']);

        DBAccess::deleteMountById($row['mount_id']);
    }

    public function test_saveCustomCreds() {
        $applicables = array('global' => true);
        DBAccess::saveMount('qazwsx2', $applicables, 'localhost', 'home', '/', 1, null, null);

        $cursor = DBAccess::getMountByProps('qazwsx2', 'localhost', 'home', '/');
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertNull($row['global_username']);
        $this->assertNull($row['global_password']);

        DBAccess::saveCustomCredentials($row['mount_id'], 'rdmUser0', 0, 'lkjh0', 'lkjhg0');
        DBAccess::saveCustomCredentials($row['mount_id'], 'rdmUser1', 0, 'lkjh', 'lkjhg');
        DBAccess::saveCustomCredentials($row['mount_id'], 'rdmUser2', 0, 'lkjh2', 'lkjhg2');

        $cursorB = DBAccess::getMountByIdWithCreds($row['mount_id'], 'rdmUser1');
        $row = $cursorB->fetchRow();
        $expected = array('mount_id' => $row['mount_id'],
                                    'mount_point' => 'qazwsx2',
                                    'mount_url' => 'localhost',
                                    'share' => 'home',
                                    'root' => '/',
                                    'use_custom_credentials' => 1,
                                    'global_username' => null,
                                    'global_password' => null,
                                    'credentials_mount_id_fk' => $row['mount_id'],
                                    'mounting_user' => 'rdmUser1',
                                    'use_global_credentials' => 0,
                                    'credential_username' => 'lkjh',
                                    'credential_password' => 'lkjhg');
        $row['use_custom_credentials'] = intval($row['use_custom_credentials']);
        $row['use_global_credentials'] = intval($row['use_global_credentials']);
        unset($row['credentials_id']);
        $this->assertEquals($expected, $row);

        $cursorC = DBAccess::getMountByIdWithCreds($row['mount_id']);
        $elements = 0;
        while ($cursorC->fetchRow() !== false) {
            $elements++;
        }
        $this->assertEquals(3, $elements);

        $cursor2 = DBAccess::getCustomCredentials($row['mount_id'], 'rdmUser1');
        $row2 = false;
        if ($cursor2 !== false) {
            $row2 = $cursor2->fetchRow();
        }
        $this->assertNotFalse($row2);
        $this->assertEquals('lkjh', $row2['credential_username']);
        $this->assertEquals('lkjhg', $row2['credential_password']);

        DBAccess::saveCustomCredentials($row['mount_id'], 'rdmUser1', 0, 'mnbv', 'mnbvcxz');

        $cursor3 = DBAccess::getCustomCredentials($row['mount_id'], 'rdmUser1');
        $row3 = false;
        if ($cursor3 !== false) {
            $row3 = $cursor3->fetchRow();
        }
        $this->assertNotFalse($row3);
        $this->assertNotEquals('lkjh', $row3['credential_username']);
        $this->assertNotEquals('lkjhg', $row3['credential_password']);
        $this->assertEquals('mnbv', $row3['credential_username']);
        $this->assertEquals('mnbvcxz', $row3['credential_password']);

        DBAccess::deleteCustomCredentialsByMountId($row['mount_id']);
        $cursor4 = DBAccess::getCustomCredentials($row['mount_id'], 'rdmUser1');
        if ($cursor4 !== false) {
            $this->assertFalse($cursor4->fetchRow());
        }
    }

    public function test_updateAdminMountType() {
        DBAccess::saveMount('qazwsx22', array('global' => 'global'), 'localhost', 'home', '/', 1, null, null);
        $cursor = DBAccess::getMountByProps('qazwsx22', 'localhost', 'home', '/');
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);

        DBAccess::saveCustomCredentials($row['mount_id'], 'rdmUser10', 0, 'lkjh', 'lkjhg');

        DBAccess::updateAdminMountCredentialsType($row['mount_id'], 0, 'qq', 'ww');
        $cursor2 = DBAccess::getMountByIdWithCreds($row['mount_id']);
        $row2 = $cursor2->fetchRow();
        $this->assertEquals(0, intval($row2['use_custom_credentials']));
        $this->assertEquals('qq', $row2['global_username']);
        $this->assertEquals('ww', $row2['global_password']);

        DBAccess::updateAdminMountCredentialsType($row['mount_id'], 2, 'qq', 'ww');
        $cursor2 = DBAccess::getMountByIdWithCreds($row['mount_id']);
        $row2 = $cursor2->fetchRow();
        $this->assertEquals(2, intval($row2['use_custom_credentials']));
        $this->assertNull($row2['global_username']);
        $this->assertNull($row2['global_password']);

        DBAccess::updateAdminMountCredentialsType($row['mount_id'], 1, 'qq', 'ww');
        $cursor2 = DBAccess::getMountByIdWithCreds($row['mount_id']);
        $row2 = $cursor2->fetchRow();
        $this->assertEquals(1, intval($row2['use_custom_credentials']));
        $this->assertNull($row2['global_username']);
        $this->assertNull($row2['global_password']);

        DBAccess::deleteCustomCredentialsByMountId($row['mount_id']);
        $cursor4 = DBAccess::getCustomCredentials($row['mount_id'], 'rdmUser1');
        if ($cursor4 !== false) {
            $this->assertFalse($cursor4->fetchRow());
        }
    }

    public function test_personalMount() {
        DBAccess::savePersonalMount('q1w2e3', 'localhost', 'home', '/', 'zaqxsw', 0, 'ytrewq', 'yuiop');
        $targetId = null;
        $cursor = DBAccess::getAllPersonalMounts();
        if ($cursor !== false) {
            while(($row = $cursor->fetchRow()) !== false) {
                if ($row['mount_point'] === 'q1w2e3' && $row['mount_url'] === 'localhost' &&
                        $row['share'] === 'home' && $row['root'] === '/') {
                    $targetId = $row['personal_mount_id'];
                    break;
                }
            }
        }
        $this->assertNotNull($targetId);

        $cursor2 = DBAccess::getPersonalMountById($targetId);
        $row2 = false;
        if ($cursor2 !== false) {
            $row2 = $cursor2->fetchRow();
        }
        $this->assertNotFalse($row2);
        $this->assertEquals('q1w2e3', $row2['mount_point']);
        $this->assertEquals('localhost', $row2['mount_url']);
        $this->assertEquals('home', $row2['share']);
        $this->assertEquals('/', $row2['root']);
        $this->assertEquals('zaqxsw', $row2['target_user']);
        $this->assertEquals('ytrewq', $row2['creds_username']);
        $this->assertEquals('yuiop', $row2['creds_password']);

        $row3 = false;
        $cursor3 = DBAccess::getPersonalMountByIdAndUser($targetId, 'zaqxsw');
        if ($cursor3 !== false) {
            $row3 = $cursor3->fetchRow();
        }
        $this->assertNotFalse($row3);
        $this->assertEquals('q1w2e3', $row3['mount_point']);
        $this->assertEquals('localhost', $row3['mount_url']);
        $this->assertEquals('home', $row3['share']);
        $this->assertEquals('/', $row3['root']);
        $this->assertEquals('zaqxsw', $row3['target_user']);
        $this->assertEquals('ytrewq', $row3['creds_username']);
        $this->assertEquals('yuiop', $row3['creds_password']);

        DBAccess::updatePersonalMountCredentials($targetId, 'asdfg', 'hjkl');
        $cursor4 = DBAccess::getPersonalMountByIdAndUser($targetId, 'zaqxsw');
        $row4 = false;
        if ($cursor4 !== false) {
            $row4 = $cursor4->fetchRow();
        }
        $this->assertNotFalse($row4);
        $this->assertEquals('q1w2e3', $row4['mount_point']);
        $this->assertEquals('localhost', $row4['mount_url']);
        $this->assertEquals('home', $row4['share']);
        $this->assertEquals('/', $row4['root']);
        $this->assertEquals('zaqxsw', $row4['target_user']);
        $this->assertNotEquals('ytrewq', $row4['creds_username']);
        $this->assertNotEquals('yuiop', $row4['creds_password']);
        $this->assertEquals('asdfg', $row4['creds_username']);
        $this->assertEquals('hjkl', $row4['creds_password']);

        DBAccess::deletePersonalMountByIdAndUser($targetId, 'zaqxsw');
        $cursor5 = DBAccess::getPersonalMountByIdAndUser($targetId, 'zaqxsw');
        if ($cursor5 !== false) {
            $this->assertFalse($cursor5->fetchRow());
        }
    }

    public function test_personalMount2() {
        DBAccess::savePersonalMount('q1w2e3r4t5', 'localhost', 'home', '/', 'rblobuser', 0, 'mkonji', 'yuiop');
        $exist = false;
        $mountId = null;
        $cursor = DBAccess::getPersonalMountsPerUser('rblobuser');
        while (($row = $cursor->fetchRow()) !== false) {
            if ($row['mount_point'] === 'q1w2e3r4t5' &&
                $row['mount_url'] === 'localhost' &&
                $row['share'] === 'home' &&
                $row['root'] === '/' &&
                $row['target_user'] === 'rblobuser') {
            $exist = true;
            $mountId = $row['personal_mount_id'];
            break;
            }
        }
        $this->assertTrue($exist);
        $this->assertEquals(0, intval($row['use_global_credentials']));

        DBAccess::updatePersonalMountCredentialsType($mountId, 2, 'zz', 'xx');
        $cursor2 = DBAccess::getPersonalMountById($mountId);
        $row2 = $cursor2->fetchRow();
        $this->assertNull($row2['creds_username']);
        $this->assertNull($row2['creds_password']);

        DBAccess::updatePersonalMountCredentialsType($mountId, 0, 'zz', 'xx');
        $cursor2 = DBAccess::getPersonalMountById($mountId);
        $row2 = $cursor2->fetchRow();
        $this->assertEquals('zz', $row2['creds_username']);
        $this->assertEquals('xx', $row2['creds_password']);

        DBAccess::deletePersonalMountById($mountId);
        $cursor2 = DBAccess::getPersonalMountById($mountId);
        $this->assertFalse($cursor2->fetchRow());
    }

    public function test_clearPersonalMounts() {
        DBAccess::clearPersonalMounts();
        $cursor = DBAccess::getAllPersonalMounts();
        if ($cursor !== false) {
            $this->assertFalse($cursor->fetchRow());
        }
    }

    public function test_globalAdminCreds() {
        DBAccess::saveAdminGlobalCredentials('ttest', 'ttuser', 'ttpass');
        $cursor = DBAccess::getAdminGlobalCredentials();
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertEquals('ttest', $row['target_user']);
        $this->assertEquals('ttuser', $row['global_username']);
        $this->assertEquals('ttpass', $row['global_password']);

        $cached = DBAccess::getAdminGlobalCredentialsCached();
        $this->assertEquals('ttest', $cached['target_user']);
        $this->assertEquals('ttuser', $cached['global_username']);
        $this->assertEquals('ttpass', $cached['global_password']);

        DBAccess::saveAdminGlobalCredentials('ttest', 'ttuser2', 'ttpass2');
        $cursor = DBAccess::getAdminGlobalCredentials();
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertEquals('ttest', $row['target_user']);
        $this->assertNotEquals('ttuser', $row['global_username']);
        $this->assertNotEquals('ttpass', $row['global_password']);
        $this->assertEquals('ttuser2', $row['global_username']);
        $this->assertEquals('ttpass2', $row['global_password']);

        $cached = DBAccess::getAdminGlobalCredentialsCached();
        $this->assertEquals('ttest', $cached['target_user']);
        $this->assertEquals('ttuser', $cached['global_username']);
        $this->assertEquals('ttpass', $cached['global_password']);
    }

    public function test_globalPersonalCreds() {
        DBAccess::savePersonalGlobalCredentials('ttest007', 'ttuser', 'ttpass');
        $cursor = DBAccess::getPersonalGlobalCredentials('ttest007');
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertEquals('ttest007', $row['target_user']);
        $this->assertEquals('ttuser', $row['global_username']);
        $this->assertEquals('ttpass', $row['global_password']);

        $cached = DBAccess::getPersonalGlobalCredentialsCached('ttest007');
        $this->assertEquals('ttest007', $cached['target_user']);
        $this->assertEquals('ttuser', $cached['global_username']);
        $this->assertEquals('ttpass', $cached['global_password']);

        DBAccess::savePersonalGlobalCredentials('ttest007', 'ttuser2', 'ttpass2');
        $cursor = DBAccess::getPersonalGlobalCredentials('ttest007');
        $row = false;
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        $this->assertNotFalse($row);
        $this->assertEquals('ttest007', $row['target_user']);
        $this->assertNotEquals('ttuser', $row['global_username']);
        $this->assertNotEquals('ttpass', $row['global_password']);
        $this->assertEquals('ttuser2', $row['global_username']);
        $this->assertEquals('ttpass2', $row['global_password']);

        $cached = DBAccess::getPersonalGlobalCredentialsCached('ttest007');
        $this->assertEquals('ttest007', $cached['target_user']);
        $this->assertEquals('ttuser', $cached['global_username']);
        $this->assertEquals('ttpass', $cached['global_password']);
    }

    public function test_getAllMountsCredentials() {
        $cursor = DBAccess::getAllMountsWithCredentials();
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        // just the first row is enough
        $this->assertArrayHasKey('mount_id', $row);
        $this->assertArrayHasKey('mount_point', $row);
        $this->assertArrayHasKey('mount_url', $row);
        $this->assertArrayHasKey('share', $row);
        $this->assertArrayHasKey('root', $row);
        $this->assertArrayHasKey('use_custom_credentials', $row);
        $this->assertArrayHasKey('global_username', $row);
        $this->assertArrayHasKey('global_password', $row);
        $this->assertArrayHasKey('credentials_mount_id_fk', $row);
        $this->assertArrayHasKey('mounting_user', $row);
        $this->assertArrayHasKey('use_global_credentials', $row);
        $this->assertArrayHasKey('credential_username', $row);
        $this->assertArrayHasKey('credential_password', $row);
    }

    public function test_getAllMountsApplicables() {
        $cursor = DBAccess::getAllMountsWithApplicables();
        if ($cursor !== false) {
            $row = $cursor->fetchRow();
        }
        // just the first row is enough
        $this->assertArrayHasKey('mount_id', $row);
        $this->assertArrayHasKey('mount_point', $row);
        $this->assertArrayHasKey('mount_url', $row);
        $this->assertArrayHasKey('share', $row);
        $this->assertArrayHasKey('root', $row);
        $this->assertArrayHasKey('use_custom_credentials', $row);
        $this->assertArrayHasKey('global_username', $row);
        $this->assertArrayHasKey('global_password', $row);
        $this->assertArrayHasKey('applicable_mount_id_fk', $row);
        $this->assertArrayHasKey('mount_type', $row);
        $this->assertArrayHasKey('mount_type_name', $row);
    }

    public function test_updateGlobalCredentials() {
        $this->assertNull(DBAccess::updateGlobalCredentials('ttest007', 'badcolumn', 'value'));
        $cursor = DBAccess::getPersonalGlobalCredentials('ttest007');
        $row = $cursor->fetchRow();
        $user = $row['global_username'];
        $pass = $row['global_password'];

        DBAccess::updateGlobalCredentials('ttest007', 'global_username', 'guser');
        $cursor = DBAccess::getPersonalGlobalCredentials('ttest007');
        $row2 = $cursor->fetchRow();
        $this->assertEquals('guser', $row2['global_username']);
        $this->assertEquals($pass, $row2['global_password']);

        DBAccess::updateGlobalCredentials('ttest007', 'global_password', 'gpass');
        $cursor = DBAccess::getPersonalGlobalCredentials('ttest007');
        $row2 = $cursor->fetchRow();
        $this->assertEquals('guser', $row2['global_username']);
        $this->assertEquals('gpass', $row2['global_password']);


        $cursor = DBAccess::getAdminGlobalCredentials();
        $row = $cursor->fetchRow();
        $user = $row['global_username'];
        $pass = $row['global_password'];

        DBAccess::updateGlobalCredentials($row['target_user'], 'global_username', 'adminuser', false);
        $cursor = DBAccess::getAdminGlobalCredentials();
        $row2 = $cursor->fetchRow();
        $this->assertEquals('adminuser', $row2['global_username']);
        $this->assertEquals($pass, $row2['global_password']);

        DBAccess::updateGlobalCredentials($row['target_user'], 'global_password', 'adminpass', false);
        $cursor = DBAccess::getAdminGlobalCredentials();
        $row2 = $cursor->fetchRow();
        $this->assertEquals('adminuser', $row2['global_username']);
        $this->assertEquals('adminpass', $row2['global_password']);
    }

    public function test_wipeOutGlobalCreds() {
        DBAccess::wipeOutGlobalCredentials();
        $cursor = DBAccess::getPersonalGlobalCredentials('ttest007');
        if ($cursor !== false) {
            $this->assertFalse($cursor->fetchRow());
        }
        $cursor2 = DBAccess::getAdminGlobalCredentials();
        if ($cursor2 !== false) {
            $this->assertFalse($cursor2->fetchRow());
        }
    }
}
